@@ -1,21 +1,23 @@
 # Copyright (C) 2013 by Yanbo Ye (yeyanbo289@gmail.com)
-# This code is part of the Biopython distribution and governed by its
-# license. Please see the LICENSE file that should have been included
-# as part of this package.
+#
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 
-""" Classes and methods for finding consensus trees.
+"""Classes and methods for finding consensus trees.
 
 This module contains a ``_BitString`` class to assist the consensus tree
 searching and some common consensus algorithms such as strict, majority rule and
 adam consensus.
 """
-from __future__ import division
 
 import random
 import itertools
 
 from ast import literal_eval
 from Bio.Phylo import BaseTree
+from Bio.Align import MultipleSeqAlignment
 
 
 class _BitString(str):
@@ -84,10 +86,8 @@ class _BitString(str):
         new_clade = BaseTree.Clade()
         new_clade.clades.extend(clade_terms)
 
-
-    Example
-    -------
-
+    Examples
+    --------
     >>> from Bio.Phylo.Consensus import _BitString
     >>> bitstr1 = _BitString('11111')
     >>> bitstr2 = _BitString('11100')
@@ -116,63 +116,66 @@ class _BitString(str):
     True
     >>> bitstr2.iscompatible(bitstr3)
     False
+
     """
 
     def __new__(cls, strdata):
-        """init from a binary string data"""
-        if (isinstance(strdata, str) and
-                len(strdata) == strdata.count('0') + strdata.count('1')):
+        """Init from a binary string data."""
+        if isinstance(strdata, str) and len(strdata) == strdata.count(
+            "0"
+        ) + strdata.count("1"):
             return str.__new__(cls, strdata)
         else:
             raise TypeError(
-                "The input should be a binary string composed of '0' and '1'")
+                "The input should be a binary string composed of '0' and '1'"
+            )
 
     def __and__(self, other):
-        selfint = literal_eval('0b' + self)
-        otherint = literal_eval('0b' + other)
+        selfint = literal_eval("0b" + self)
+        otherint = literal_eval("0b" + other)
         resultint = selfint & otherint
         return _BitString(bin(resultint)[2:].zfill(len(self)))
 
     def __or__(self, other):
-        selfint = literal_eval('0b' + self)
-        otherint = literal_eval('0b' + other)
+        selfint = literal_eval("0b" + self)
+        otherint = literal_eval("0b" + other)
         resultint = selfint | otherint
         return _BitString(bin(resultint)[2:].zfill(len(self)))
 
     def __xor__(self, other):
-        selfint = literal_eval('0b' + self)
-        otherint = literal_eval('0b' + other)
+        selfint = literal_eval("0b" + self)
+        otherint = literal_eval("0b" + other)
         resultint = selfint ^ otherint
         return _BitString(bin(resultint)[2:].zfill(len(self)))
 
     def __rand__(self, other):
-        selfint = literal_eval('0b' + self)
-        otherint = literal_eval('0b' + other)
+        selfint = literal_eval("0b" + self)
+        otherint = literal_eval("0b" + other)
         resultint = otherint & selfint
         return _BitString(bin(resultint)[2:].zfill(len(self)))
 
     def __ror__(self, other):
-        selfint = literal_eval('0b' + self)
-        otherint = literal_eval('0b' + other)
+        selfint = literal_eval("0b" + self)
+        otherint = literal_eval("0b" + other)
         resultint = otherint | selfint
         return _BitString(bin(resultint)[2:].zfill(len(self)))
 
     def __rxor__(self, other):
-        selfint = literal_eval('0b' + self)
-        otherint = literal_eval('0b' + other)
+        selfint = literal_eval("0b" + self)
+        otherint = literal_eval("0b" + other)
         resultint = otherint ^ selfint
         return _BitString(bin(resultint)[2:].zfill(len(self)))
 
     def __repr__(self):
-        return '_BitString(' + str.__repr__(self) + ')'
+        return "_BitString(" + str.__repr__(self) + ")"
 
     def index_one(self):
-        """Return a list of positions where the element is '1'"""
-        return [i for i, n in enumerate(self) if n == '1']
+        """Return a list of positions where the element is '1'."""
+        return [i for i, n in enumerate(self) if n == "1"]
 
     def index_zero(self):
-        """Return a list of positions where the element is '0'"""
-        return [i for i, n in enumerate(self) if n == '0']
+        """Return a list of positions where the element is '0'."""
+        return [i for i, n in enumerate(self) if n == "0"]
 
     def contains(self, other):
         """Check if current bitstr1 contains another one bitstr2.
@@ -185,9 +188,10 @@ class _BitString(str):
 
         Be careful, "011011" also contains "000000". Actually, all _BitString
         objects contain all-zero _BitString of the same length.
+
         """
         xorbit = self ^ other
-        return (xorbit.count('1') == self.count('1') - other.count('1'))
+        return xorbit.count("1") == self.count("1") - other.count("1")
 
     def independent(self, other):
         """Check if current bitstr1 is independent of another one bitstr2.
@@ -199,22 +203,21 @@ class _BitString(str):
         of the same length.
         """
         xorbit = self ^ other
-        return (xorbit.count('1') == self.count('1') + other.count('1'))
+        return xorbit.count("1") == self.count("1") + other.count("1")
 
     def iscompatible(self, other):
         """Check if current bitstr1 is compatible with another bitstr2.
 
         Two conditions are considered as compatible:
+         1. bitstr1.contain(bitstr2) or vice versa;
+         2. bitstr1.independent(bitstr2).
 
-        1. bitstr1.contain(bitstr2) or vise versa;
-        2. bitstr1.independent(bitstr2).
         """
-        return (self.contains(other) or other.contains(self) or
-                self.independent(other))
+        return self.contains(other) or other.contains(self) or self.independent(other)
 
     @classmethod
     def from_bool(cls, bools):
-        return cls(''.join(map(str, map(int, bools))))
+        return cls("".join(map(str, map(int, bools))))
 
 
 def strict_consensus(trees):
@@ -223,24 +226,25 @@ def strict_consensus(trees):
     :Parameters:
         trees : iterable
             iterable of trees to produce consensus tree.
+
     """
     trees_iter = iter(trees)
     first_tree = next(trees_iter)
 
     terms = first_tree.get_terminals()
-    bitstr_counts, tree_count = _count_clades(
-        itertools.chain([first_tree], trees_iter))
+    bitstr_counts, tree_count = _count_clades(itertools.chain([first_tree], trees_iter))
 
     # Store bitstrs for strict clades
-    strict_bitstrs = [bitstr for bitstr, t in bitstr_counts.items()
-                      if t[0] == tree_count]
-    strict_bitstrs.sort(key=lambda bitstr: bitstr.count('1'), reverse=True)
+    strict_bitstrs = [
+        bitstr for bitstr, t in bitstr_counts.items() if t[0] == tree_count
+    ]
+    strict_bitstrs.sort(key=lambda bitstr: bitstr.count("1"), reverse=True)
     # Create root
     root = BaseTree.Clade()
-    if strict_bitstrs[0].count('1') == len(terms):
+    if strict_bitstrs[0].count("1") == len(terms):
         root.clades.extend(terms)
     else:
-        raise ValueError('Taxons in provided trees should be consistent')
+        raise ValueError("Taxons in provided trees should be consistent")
     # make a bitstr to clades dict and store root clade
     bitstr_clades = {strict_bitstrs[0]: root}
     # create inner clades
@@ -254,8 +258,7 @@ def strict_consensus(trees):
                 # remove old bitstring
                 del bitstr_clades[bs]
                 # update clade childs
-                new_childs = [child for child in c.clades
-                              if child not in clade_terms]
+                new_childs = [child for child in c.clades if child not in clade_terms]
                 c.clades = new_childs
                 # set current clade as child of c
                 c.clades.append(clade)
@@ -282,25 +285,25 @@ def majority_consensus(trees, cutoff=0):
     :Parameters:
         trees : iterable
             iterable of trees to produce consensus tree.
+
     """
     tree_iter = iter(trees)
     first_tree = next(tree_iter)
 
     terms = first_tree.get_terminals()
-    bitstr_counts, tree_count = _count_clades(
-        itertools.chain([first_tree], tree_iter))
+    bitstr_counts, tree_count = _count_clades(itertools.chain([first_tree], tree_iter))
 
     # Sort bitstrs by descending #occurrences, then #tips, then tip order
-    bitstrs = sorted(bitstr_counts.keys(),
-                     key=lambda bitstr: (bitstr_counts[bitstr][0],
-                                         bitstr.count('1'),
-                                         str(bitstr)),
-                     reverse=True)
+    bitstrs = sorted(
+        bitstr_counts.keys(),
+        key=lambda bitstr: (bitstr_counts[bitstr][0], bitstr.count("1"), str(bitstr)),
+        reverse=True,
+    )
     root = BaseTree.Clade()
-    if bitstrs[0].count('1') == len(terms):
+    if bitstrs[0].count("1") == len(terms):
         root.clades.extend(terms)
     else:
-        raise ValueError('Taxons in provided trees should be consistent')
+        raise ValueError("Taxons in provided trees should be consistent")
     # Make a bitstr-to-clades dict and store root clade
     bitstr_clades = {bitstrs[0]: root}
     # create inner clades
@@ -315,11 +318,10 @@ def majority_consensus(trees, cutoff=0):
         clade.clades.extend(clade_terms)
         clade.confidence = confidence
         clade.branch_length = branch_length_sum / count_in_trees
-        bsckeys = sorted(bitstr_clades, key=lambda bs: bs.count('1'),
-                         reverse=True)
+        bsckeys = sorted(bitstr_clades, key=lambda bs: bs.count("1"), reverse=True)
 
         # check if current clade is compatible with previous clades and
-        # record it's possible parent and child clades.
+        # record its possible parent and child clades.
         compatible = True
         parent_bitstr = None
         child_bitstrs = []  # multiple independent childs
@@ -333,8 +335,11 @@ def majority_consensus(trees, cutoff=0):
                 parent_bitstr = bs
             # assign the closest descendant as its child
             # the largest and independent clades
-            if (bitstr.contains(bs) and bs != bitstr and
-                    all(c.independent(bs) for c in child_bitstrs)):
+            if (
+                bitstr.contains(bs)
+                and bs != bitstr
+                and all(c.independent(bs) for c in child_bitstrs)
+            ):
                 child_bitstrs.append(bs)
         if not compatible:
             continue
@@ -343,8 +348,9 @@ def majority_consensus(trees, cutoff=0):
             # insert current clade; remove old bitstring
             parent_clade = bitstr_clades.pop(parent_bitstr)
             # update parent clade childs
-            parent_clade.clades = [c for c in parent_clade.clades
-                                   if c not in clade_terms]
+            parent_clade.clades = [
+                c for c in parent_clade.clades if c not in clade_terms
+            ]
             # set current clade as child of parent_clade
             parent_clade.clades.append(clade)
             # update bitstring
@@ -363,32 +369,34 @@ def majority_consensus(trees, cutoff=0):
             clade.clades = [c for c in clade.clades if c not in remove_terms]
         # put new clade
         bitstr_clades[bitstr] = clade
-        if ((len(bitstr_clades) == len(terms) - 1) or
-                (len(bitstr_clades) == len(terms) - 2 and len(root.clades) == 3)):
+        if (len(bitstr_clades) == len(terms) - 1) or (
+            len(bitstr_clades) == len(terms) - 2 and len(root.clades) == 3
+        ):
             break
     return BaseTree.Tree(root=root)
 
 
 def adam_consensus(trees):
-    """Search Adam Consensus tree from multiple trees
+    """Search Adam Consensus tree from multiple trees.
 
     :Parameters:
         trees : list
             list of trees to produce consensus tree.
+
     """
     clades = [tree.root for tree in trees]
     return BaseTree.Tree(root=_part(clades), rooted=True)
 
 
 def _part(clades):
-    """recursive function of adam consensus algorithm"""
+    """Recursive function for Adam Consensus algorithm (PRIVATE)."""
     new_clade = None
     terms = clades[0].get_terminals()
     term_names = [term.name for term in terms]
     if len(terms) == 1 or len(terms) == 2:
         new_clade = clades[0]
     else:
-        bitstrs = set([_BitString('1' * len(terms))])
+        bitstrs = {_BitString("1" * len(terms))}
         for clade in clades:
             for child in clade.clades:
                 bitstr = _clade_to_bitstr(child, term_names)
@@ -411,7 +419,7 @@ def _part(clades):
                 # bitstrs = bitstrs | to_add
                 bitstrs ^= to_remove
                 if to_add:
-                    for ta in sorted(to_add, key=lambda bs: bs.count('1')):
+                    for ta in sorted(to_add, key=lambda bs: bs.count("1")):
                         independent = True
                         for bs in bitstrs:
                             if not ta.independent(bs):
@@ -440,7 +448,7 @@ def _part(clades):
 
 
 def _sub_clade(clade, term_names):
-    """extract a compatible subclade that only contains the given terminal names"""
+    """Extract a compatible subclade that only contains the given terminal names (PRIVATE)."""
     term_clades = [clade.find_any(name) for name in term_names]
     sub_clade = clade.common_ancestor(term_clades)
     if len(term_names) != sub_clade.count_terminals():
@@ -451,8 +459,7 @@ def _sub_clade(clade, term_names):
                 continue
             childs = set(c.find_clades(terminal=True)) & set(term_clades)
             if childs:
-                for tc in temp_clade.find_clades(terminal=False,
-                                                 order="preorder"):
+                for tc in temp_clade.find_clades(terminal=False, order="preorder"):
                     tc_childs = set(tc.clades)
                     tc_new_clades = tc_childs - childs
                     if childs.issubset(tc_childs) and tc_new_clades:
@@ -465,7 +472,7 @@ def _sub_clade(clade, term_names):
 
 
 def _count_clades(trees):
-    """Count distinct clades (different sets of terminal names) in the trees.
+    """Count distinct clades (different sets of terminal names) in the trees (PRIVATE).
 
     Return a tuple first a dict of bitstring (representing clade) and a tuple of its count of
     occurrences and sum of branch length for that clade, second the number of trees processed.
@@ -473,6 +480,7 @@ def _count_clades(trees):
     :Parameters:
         trees : iterable
             An iterable that returns the trees to count
+
     """
     bitstrs = {}
     tree_count = 0
@@ -502,9 +510,9 @@ def get_support(target_tree, trees, len_trees=None):
         len_trees : int
             optional count of replicates in trees. len_trees must be provided
             when len(trees) is not a valid operation.
+
     """
-    term_names = sorted(term.name
-                        for term in target_tree.find_clades(terminal=True))
+    term_names = sorted(term.name for term in target_tree.find_clades(terminal=True))
     bitstrs = {}
 
     size = len_trees
@@ -512,9 +520,11 @@ def get_support(target_tree, trees, len_trees=None):
         try:
             size = len(trees)
         except TypeError:
-            raise TypeError("Trees does not support len(trees), "
-                            "you must provide the number of replicates in trees "
-                            "as the optional parameter len_trees.")
+            raise TypeError(
+                "Trees does not support len(trees), "
+                "you must provide the number of replicates in trees "
+                "as the optional parameter len_trees."
+            ) from None
 
     for clade in target_tree.find_clades(terminal=False):
         bitstr = _clade_to_bitstr(clade, term_names)
@@ -530,13 +540,14 @@ def get_support(target_tree, trees, len_trees=None):
 
 
 def bootstrap(msa, times):
-    """Generate bootstrap replicates from a multiple sequence alignment object
+    """Generate bootstrap replicates from a multiple sequence alignment (OBSOLETE).
 
     :Parameters:
         msa : MultipleSeqAlignment
             multiple sequence alignment to generate replicates.
         times : int
             number of bootstrap times.
+
     """
     length = len(msa[0])
     i = 0
@@ -546,58 +557,72 @@ def bootstrap(msa, times):
         for j in range(length):
             col = random.randint(0, length - 1)
             if not item:
-                item = msa[:, col:col + 1]
+                item = msa[:, col : col + 1]
             else:
-                item += msa[:, col:col + 1]
+                item += msa[:, col : col + 1]
         yield item
 
 
-def bootstrap_trees(msa, times, tree_constructor):
+def bootstrap_trees(alignment, times, tree_constructor):
     """Generate bootstrap replicate trees from a multiple sequence alignment.
 
     :Parameters:
-        msa : MultipleSeqAlignment
+        alignment : Alignment or MultipleSeqAlignment object
             multiple sequence alignment to generate replicates.
         times : int
             number of bootstrap times.
         tree_constructor : TreeConstructor
             tree constructor to be used to build trees.
+
     """
-    msas = bootstrap(msa, times)
-    for aln in msas:
-        tree = tree_constructor.build_tree(aln)
-        yield tree
+    if isinstance(alignment, MultipleSeqAlignment):
+        length = len(alignment[0])
+        for i in range(times):
+            bootstrapped_alignment = None
+            for j in range(length):
+                col = random.randint(0, length - 1)
+                if bootstrapped_alignment is None:
+                    bootstrapped_alignment = alignment[:, col : col + 1]
+                else:
+                    bootstrapped_alignment += alignment[:, col : col + 1]
+            tree = tree_constructor.build_tree(alignment)
+            yield tree
+    else:
+        n, m = alignment.shape
+        for i in range(times):
+            cols = [random.randint(0, m - 1) for j in range(m)]
+            tree = tree_constructor.build_tree(alignment[:, cols])
+            yield tree
 
 
-def bootstrap_consensus(msa, times, tree_constructor, consensus):
-    """Consensus tree of a series of bootstrap trees for a multiple sequence alignment
+def bootstrap_consensus(alignment, times, tree_constructor, consensus):
+    """Consensus tree of a series of bootstrap trees for a multiple sequence alignment.
 
     :Parameters:
-        msa : MultipleSeqAlignment
+        alignment : Alignment or MultipleSeqAlignment object
             Multiple sequence alignment to generate replicates.
         times : int
             Number of bootstrap times.
         tree_constructor : TreeConstructor
             Tree constructor to be used to build trees.
         consensus : function
-            Consensus method in this module: `strict_consensus`,
-            `majority_consensus`, `adam_consensus`.
+            Consensus method in this module: ``strict_consensus``,
+            ``majority_consensus``, ``adam_consensus``.
+
     """
-    trees = bootstrap_trees(msa, times, tree_constructor)
-    tree = consensus(list(trees))
+    trees = bootstrap_trees(alignment, times, tree_constructor)
+    tree = consensus(trees)
     return tree
 
 
 def _clade_to_bitstr(clade, tree_term_names):
-    """Create a BitString representing a clade, given ordered tree taxon names."""
-    clade_term_names = set(term.name for term in
-                           clade.find_clades(terminal=True))
-    return _BitString.from_bool((name in clade_term_names)
-                                for name in tree_term_names)
+    """Create a BitString representing a clade, given ordered tree taxon names (PRIVATE)."""
+    clade_term_names = {term.name for term in clade.find_clades(terminal=True)}
+    return _BitString.from_bool((name in clade_term_names) for name in tree_term_names)
 
 
 def _tree_to_bitstrs(tree):
-    """Create a dict of a tree's clades to corresponding BitStrings."""
+    """Create a dict of a tree's clades to corresponding BitStrings (PRIVATE)."""
     clades_bitstrs = {}
     term_names = [term.name for term in tree.find_clades(terminal=True)]
     for clade in tree.find_clades(terminal=False):
@@ -607,10 +632,11 @@ def _tree_to_bitstrs(tree):
 
 
 def _bitstring_topology(tree):
-    """Generates a branch length dict for a tree, keyed by BitStrings.
+    """Generate a branch length dict for a tree, keyed by BitStrings (PRIVATE).
 
     Create a dict of all clades' BitStrings to the corresponding branch
-    lengths (rounded to 5 decimal places)."""
+    lengths (rounded to 5 decimal places).
+    """
     bitstrs = {}
     for clade, bitstr in _tree_to_bitstrs(tree).items():
         bitstrs[bitstr] = round(clade.branch_length or 0.0, 5)
@@ -618,11 +644,12 @@ def _bitstring_topology(tree):
 
 
 def _equal_topology(tree1, tree2):
-    """Are two trees are equal in terms of topology and branch lengths.
+    """Are two trees are equal in terms of topology and branch lengths (PRIVATE).
 
     (Branch lengths checked to 5 decimal places.)
     """
-    term_names1 = set(term.name for term in tree1.find_clades(terminal=True))
-    term_names2 = set(term.name for term in tree2.find_clades(terminal=True))
-    return ((term_names1 == term_names2) and
-            (_bitstring_topology(tree1) == _bitstring_topology(tree2)))
+    term_names1 = {term.name for term in tree1.find_clades(terminal=True)}
+    term_names2 = {term.name for term in tree2.find_clades(terminal=True)}
+    return (term_names1 == term_names2) and (
+        _bitstring_topology(tree1) == _bitstring_topology(tree2)
+    )
