@@ -1,7 +1,9 @@
 # Copyright 2012 by Wibowo Arindrarto.  All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
+#
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 
 """Bio.SearchIO support for BLAST+ output formats.
 
@@ -18,7 +20,7 @@ Bio.SearchIO.BlastIO was tested on the following BLAST+ flavors and versions:
 You should also be able to parse outputs from a local BLAST+ search or from
 NCBI's web interface. Although the module was not tested against all BLAST+,
 it should still be able to parse these other versions' outputs. Please submit
-a bug report if you stumble upon an unparseable file.
+a bug report if you stumble upon an unparsable file.
 
 Some output formats from the BLAST legacy suite (BLAST+'s predecessor) may
 still be parsed by this module. However, results are not guaranteed. You may
@@ -263,9 +265,9 @@ blast-tab provides the following attributes for each SearchIO objects:
 |             +-------------------+--------------+
 |             | gap_num           | gaps         |
 |             +-------------------+--------------+
-|             | ident_pct         | nident       |
+|             | ident_num         | nident       |
 |             +-------------------+--------------+
-|             | ident_num         | pident       |
+|             | ident_pct         | pident       |
 |             +-------------------+--------------+
 |             | mismatch_num      | mismatch     |
 |             +-------------------+--------------+
@@ -410,8 +412,8 @@ from .blast_tab import BlastTabParser, BlastTabIndexer, BlastTabWriter
 from .blast_xml import BlastXmlParser, BlastXmlIndexer, BlastXmlWriter
 from .blast_text import BlastTextParser
 
-
 # if not used as a module, run the doctest
 if __name__ == "__main__":
     from Bio._utils import run_doctest
+
     run_doctest()
