@@ -6,10 +6,10 @@
 """Abstract base classes for the SearchIO object model."""
 
 
-from Bio._utils import getattr_str, trim_str
+from Bio.SearchIO._utils import getattr_str
 
 
-class _BaseSearchObject(object):
+class _BaseSearchObject:
     """Abstract class for SearchIO objects."""
 
     _NON_STICKY_ATTRS = ()
@@ -41,13 +41,11 @@ class _BaseHSP(_BaseSearchObject):
         """Print the alignment header info (PRIVATE)."""
         lines = []
         # set query id line
-        qid_line = trim_str(
-            "      Query: %s %s" % (self.query_id, self.query_description), 80, "..."
-        )
+        qid_line = f"      Query: {self.query_id} {self.query_description}"
+        qid_line = qid_line[:77] + "..." if len(qid_line) > 80 else qid_line
         # set hit id line
-        hid_line = trim_str(
-            "        Hit: %s %s" % (self.hit_id, self.hit_description), 80, "..."
-        )
+        hid_line = f"        Hit: {self.hit_id} {self.hit_description}"
+        hid_line = hid_line[:77] + "..." if len(hid_line) > 80 else hid_line
         lines.append(qid_line)
         lines.append(hid_line)
 
@@ -64,7 +62,7 @@ class _BaseHSP(_BaseSearchObject):
         except ValueError:
             qstrand = self.query_strand_all[0]
             hstrand = self.hit_strand_all[0]
-        lines.append("Query range: [%s:%s] (%r)" % (query_start, query_end, qstrand))
-        lines.append("  Hit range: [%s:%s] (%r)" % (hit_start, hit_end, hstrand))
+        lines.append(f"Query range: [{query_start}:{query_end}] ({qstrand!r})")
+        lines.append(f"  Hit range: [{hit_start}:{hit_end}] ({hstrand!r})")
 
         return "\n".join(lines)
